#!/usr/bin/python2
# -*- coding: utf-8 -*-

# Import modules for CGI handling
import cgi
import datetime
import os
import io
import re
import distutils.dir_util
import collections
import json

# Create instance of FieldStorage
form = cgi.FieldStorage()

ip_address = os.environ.get('REMOTE_ADDR')
modality = form.getvalue('modality', '').lower()
uuid = form.getvalue('uuid')
uuid_short = uuid[:8] if uuid else 'UUID'

resultsdir = "/var/www/results/survey/"


def init():
    fields = get_fields(form)

    try:
        if not can_submit(fields):
            write_file(fields, True)
            return respond(True)

        write_file(fields)
        respond(True)

    except Exception as e:
        respond(False, "403 Forbidden", str(e))

def is_online_modality():
    return not modality in ('vt', 'ilt')


def get_fields(fieldStorage):
    """ Get a plain dictionary rather than the '.value' system used by the
    cgi module's native fieldStorage class. """
    params = {}

    for key in fieldStorage.keys():
        value = fieldStorage[key].value
        if len(value):
            params[key] = value

    # Store IP address
    params['ip'] = ip_address

    # Trim leading and trailing spaces for each item, if any
    for key, value in params.iteritems():
        search = re.compile('\s+', re.MULTILINE)
        text = search.sub(' ', value.strip()) if value else value
        params[key] = text

    # Create an ordered dictionary for better organization
    return collections.OrderedDict(
        [(k, params[k]) for k in sorted(params.keys())]
    )


def get_filename(fields, duplicate=False):
    """ Return the filename based on the user's modality """
    today = datetime.date.today()
    year = today.year

    # The full date formatted as 20170324
    date_full = today.strftime('%Y%m%d')
    # The date including the week number formatted as 2017(12)
    date_week = '%s(%s)' % (year, today.isocalendar()[1])

    # Set the date string depending on whether not not the student is coming
    # from an online modality
    date = date_week if is_online_modality() else date_full

    return "%s-survey-%s-%s-%s-%s-%s%s.txt" % (
        date,
        fields.get('course', 'COURSE'),
        fields.get('modality', 'MODALITY'),
        fields.get('subscript', 'SUBSCRIPTION'),
        fields.get('instructor', 'INSTRUCTOR'),
        uuid_short,
        '_duplicates' if duplicate else ''
    )


def can_submit(fields):
    """
    Check if the user has already submitted a survey for this course today
    """

    filename = get_filename(fields)
    filepath = '%s%s' % (resultsdir, filename)
    search = "ip=%s\n" % ip_address
    ilt_user = modality == 'ilt'

    file_exists = os.path.exists(filepath)

    if ilt_user and file_exists and search in io.open(
        filepath, 'r', encoding='utf-8', newline=None
    ).read():
        return False

    return True


# write results to file
def write_file(fields, duplicate=False):
    """ Write the file out to the server """
    filename = get_filename(fields, duplicate)

    # Create survey results directory if it doesn't exist
    distutils.dir_util.mkpath(resultsdir)

    with io.open(
        '%s%s' % (resultsdir, filename), 'a', encoding='utf-8'
    ) as fileOutput:
        for k, v in fields.iteritems():
            v = v.decode('utf-8')

            fileOutput.write(
                (u'%s=%s' % (k, v)).strip()
            )

            fileOutput.write(u'\n')

        fileOutput.write(u'=')
        fileOutput.write(u'\n')


def respond(accept=True, status="200 OK", msg='success'):
    print "Access-Control-Allow-Origin: *"
    print "Content-type: application/json"
    print "Status: %s" % status
    print
    response = {
        'result': msg
    }
    print(json.JSONEncoder().encode(response))


init()
